/*
 * Copyright (c) 1994  Harris Computer Systems Division
 *
 * The X Consortium, and any party obtaining a copy of these files from the X
 * Consortium, directly or indirectly, is granted, free of charge, a full and
 * unrestricted irrevocable, world-wide, paid up, royalty-free, nonexclusive
 * right and license to deal in this software and documentation files (the
 * "Software"), including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons who receive copies from any such party to do so.
 * This license includes without limitation a license to do the foregoing
 * actions under any patents of the party supplying this software to the X
 * Consortium.
 * 
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
 * THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
 * OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
 * TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
 * PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
 * REPAIR OR CORRECTION.
 * 
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
 * ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
 * THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
 * GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
 * USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF
 * DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
 * PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS),
 * EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGES.
 */

/*==========================================================================*/
/*                              Calendar Labels:                            */
/*==========================================================================*/

extern Boolean european;

static char* languages[] = {
   /* This order must match the following arrays. */
   "english",
   "german",
   "spanish",
   "french",
   "portuguese",
   "irish",
   "italian",
   NULL
};

static char* weekday_names[][8] = {
   {
      /* ENGLISH */
      "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday",
      "Friday", "Saturday", NULL
   },
   {
      /* GERMAN */
      "Sonntag", "Montag", "Dienstag", "Mittwoch", "Donnerstag",
      "Freitag", "Samstag", NULL
   },
   {
      /* SPANISH */
      "Domingo", "Lunes", "Martes", "Mi\351rcoles", "Jueves", "Viernes",
      "S\341bado",  NULL
   },
   {
      /* FRENCH */
      "Dimanche", "Lundi", "Mardi", "Mercredi", "Jeudi", "Vendredi",
      "Samedi", NULL
   },
   {
      /* PORTUGUESE */
      "Domingo", "Segunda", "Ter\347a", "Quarta", "Quinta", "Sexta",
      "S\341bado", NULL
   },
   {
      /* IRISH */
      "Domhnach", "Luan", "M\341irt", "C\351adaoin", "D\351ardaoin", "Aoine",
      "Satharn", NULL
   },
   {
      /* ITALIAN */
      "Domenica", "Luned\354", "Marted\354", "Mercoled\354", "Gioved\354",
      "Venerd\354", "Sabato", NULL
   }
   
};

static char* month_names[][13] = {
   {
      /* ENGLISH */
      "January", "February", "March", "April", "May", "June", "July",
      "August", "September", "October", "November", "December", NULL
   },
   {
      /* GERMAN */
      "Januar", "Februar", "M\344rz", "April", "Mai", "Juni", "Juli",
      "August", "September", "Oktober", "November", "Dezember", NULL
   },
   {
      /* SPANISH */
      "Enero", "Febrero", "Marzo", "Abril", "Mayo", "Junio", "Julio",
      "Agosto", "Septiembre", "Octubre", "Noviembre", "Diciembre", NULL
   },
   {
      /* FRENCH */
      "Janvier", "F\351vrier", "Mars", "Avril", "Mai", "Juin", "Juillet",
      "Ao\373t", "Septembre", "Octobre", "Novembre", "D\351cembre", NULL
   },
   {
      /* PORTUGUESE */
      "Janeiro", "Fevereiro", "Mar\347o", "Abril", "Maio", "Junho", "Julho",
      "Agosto", "Setembro", "Outubro", "Novembro", "Dezembro", NULL
   },
   {
      /* IRISH */
      "Ean\341ir", "Feabhra", "Marta", "Aibre\341n", "Bealtaine", "Meitheamh",
      "I\372il", "L\372nasa", "Me\341n F\363mhair", "Deireadh F\363mhair",
      "Samhain", "Nollaig", NULL
   },
   {
      /* ITALIAN */
      "Gennaio", "Febbraio", "Marzo", "Aprile", "Maggio", "Giugno", "Luglio",
      "Agosto", "Settembre", "Ottobre", "Novembre", "Dicembre", NULL
   }
};




/*==========================================================================*/
/*                                Button Labels:                            */
/*==========================================================================*/

static char* day_label[] = {
   /* ENGLISH */        "Day",
   /* GERMAN */         "Tag",
   /* SPANISH */        "Dia",
   /* FRENCH */         "Jour",
   /* PORTUGUESE */     "Dia",
   /* IRISH */          "L\341",
   /* ITALIAN */        "Giorno"
};

static char* week_label[] = {
   /* This order must match the following arrays. */
   /* ENGLISH */        "Week",
   /* GERMAN */         "Woche",
   /* SPANISH */        "Sema\361a",
   /* FRENCH */         "Semaine",
   /* PORTUGUESE */     "Semana",
   /* IRISH */          "Week",
   /* ITALIAN */        "Settimana"
};

static char* entry_label[] = {
   /* ENGLISH */        "Entry",
   /* GERMAN */         "Eintrag",
   /* SPANISH */        "Entrada",
   /* FRENCH */         "Entr\351e",
   /* PORTUGUESE */     "Registo",
   /* IRISH */          "Iontr\341il",
   /* ITALIAN */        "Voce Calendario"
};

static char* month_label[] = {
   /* ENGLISH */        "Month",
   /* GERMAN */         "Monat",
   /* SPANISH */        "Mes",
   /* FRENCH */         "Mois",
   /* PORTUGUESE */     "M\352s",
   /* IRISH */          "M\355",
   /* ITALIAN */        "Mese"
};

static char* year_label[] = {
   /* ENGLISH */        "Year",
   /* GERMAN */         "Jahr",
   /* SPANISH */        "A\361o",
   /* FRENCH */         "Ann\351e",
   /* PORTUGUESE */     "Ano",
   /* IRISH */          "Bliain",
   /* ITALIAN */        "Anno"
};


/*==========================================================================*/
/*                                Menu Labels:                              */
/*==========================================================================*/

static char* option_label[] = {
   /* ENGLISH */        "Option",
   /* GERMAN */         "Optionen",
   /* SPANISH */        "Opci\363n",
   /* FRENCH */         "Option",
   /* PORTUGUESE */     "Op\347\343o",
   /* IRISH */          "Rogha",
   /* ITALIAN */        "Opzione"
};

static char* goto_today_label[] = {
   /* ENGLISH */        "Go To Today",
   /* GERMAN */         "Gehe nach Heute",
   /* SPANISH */        "Vaya A Fecha De Hoy",
   /* FRENCH */         "Revenir \340 Aujourd'hui",
   /* PORTUGUESE */     "Ir Para o Dia de Hoje",
   /* IRISH */          "T\351igh Go Dt\355 Inni\372",
   /* ITALIAN */        "Alla Data Odierna"
};

static char* goto_date_label[] = {
   /* ENGLISH */        "Go To Date...",
   /* GERMAN */         "Zeige Datum...",
   /* SPANISH */        "Vaya A Fecha...",
   /* FRENCH */         "Sp\351cifier la Date...",
   /* PORTUGUESE */     "Ir Para Data...",
   /* IRISH */          "T\351igh Go Dt\355 D\341ta...",
   /* ITALIAN */        "Vai Alla Data..."
};

static char* set_backward_label[] = {
   /* ENGLISH */        "Previous Items, How Many...",
   /* GERMAN */         "Anzahl der vorhergehenden Eintr\344ge...",
   /* SPANISH */        "Art\355culos Anteriores, Cu\341ntos",
   /* FRENCH */         "Entr\351es Pr\351c\351dentes, Combien...",
   /* PORTUGUESE */     "Artigos Anteriores, Quanto...",
   /* IRISH */          "Previous Items, How many...",
   /* ITALIAN */        "Articoli Precedenti, Quanto..."
};

static char* set_forward_label[] = {
   /* ENGLISH */        "Following Items, How Many...",
   /* GERMAN */         "Anzahl der folgenden Eintr\344ge...",
   /* SPANISH */        "Art\355culos Posteriores, Cu\341ntos...",
   /* FRENCH */         "Entr\351es Suivantes, Combien...",
   /* PORTUGUESE */     "Artigos Posteriores, Quanto...",
   /* IRISH */          "Following Items, How Many...",
   /* ITALIAN */        "Articoli Seguenti, Quanto..."
};

static char* show_diary_label[] = {
   /* ENGLISH */        "Show Diary",
   /* GERMAN */         "Zeige Eintr\344ge",
   /* SPANISH */        "Muestre Diario",
   /* FRENCH */         "Afficher Agenda",
   /* PORTUGUESE */     "Mostrar Di\341rio",
   /* IRISH */          "Foilsigh An Dhialann",
   /* ITALIAN */        "Mostra Diario"
};

static char* hide_diary_label[] = {
   /* ENGLISH */        "Hide Diary",
   /* GERMAN */         "Fenster Schlie\337en",
   /* SPANISH */        "Esconda Diario",
   /* FRENCH */         "Masquer Agenda",
   /* PORTUGUESE */     "Esconder Di\341rio",
   /* IRISH */          "Folaigh An Dhialann",
   /* ITALIAN */        "Nascondi Diario"
};

static char* show_itemlist_label[] = {
   /* ENGLISH */        "Show Item List",
   /* GERMAN */         "Zeige die Liste der Eintr\344ge",
   /* SPANISH */        "Muestre Lista de los Art\355culos ",
   /* FRENCH */         "Afficher Liste d'Entr\351es",
   /* PORTUGUESE */     "Mostre a Lista dos Artigos",
   /* IRISH */          "Foilsigh Item List...",
   /* ITALIAN */        "Mostra Lista degli Articoli"
};

static char* hide_itemlist_label[] = {
   /* ENGLISH */        "Hide the Item List",
   /* GERMAN */         "Liste der Eintr\344ge ausblenden",
   /* SPANISH */        "Oculte la Lista de los Art\355culos",
   /* FRENCH */         "Masquer Liste d'Entr\351es",
   /* PORTUGUESE */     "Esconder a lista dos Artigos",
   /* IRISH */          "Folaigh Item List...",
   /* ITALIAN */        "Nascondi la Lista degli Articoli"
};

static char* show_calendar_label[] = {
   /* ENGLISH */        "Show Calendar",
   /* GERMAN */         "Zeige Kalender",
   /* SPANISH */        "Muestre Calendario",
   /* FRENCH */         "Afficher Le Calendrier",
   /* PORTUGUESE */     "Mostrar Calend\225rio",
   /* IRISH */          "Foilsigh an F\351ilire",
   /* ITALIAN */        "Mostra Calendario"
};

static char* hide_calendar_label[] = {
   /* ENGLISH */        "Hide Calendar",
   /* GERMAN */         "Kalender Schlie\337en",
   /* SPANISH */        "Esconda Calendario",
   /* FRENCH */         "Masquer Calendrier",
   /* PORTUGUESE */     "Esconder Calend\341rio",
   /* IRISH */          "Folaigh an Fe\355lire",
   /* ITALIAN */        "Nascondi Calendario"
};

static char* one_month_label[] = {
   /* ENGLISH */        "One Month Calendar",
   /* GERMAN */         "Monatskalender",
   /* SPANISH */        "Calendario de Un Mes",
   /* FRENCH */         "Calendrier du Mois",
   /* PORTUGUESE */     "Calend\341rio de um S\363 M\352s",
   /* IRISH */          "F\351ilire M\355osa",
   /* ITALIAN */        "Calendario Formato Mensile"
};

static char* full_year_label[] = {
   /* ENGLISH */        "Full Year Calendar",
   /* GERMAN */         "Jahreskalender",
   /* SPANISH */        "Calendario de A\361o Completo",
   /* FRENCH */         "Calendrier Annuel",
   /* PORTUGUESE */     "Calend\341rio do Ano Inteiro",
   /* IRISH */          "Fe\355lire Bliana",
   /* ITALIAN */        "Calendario di Tutto l'Anno"
};

static char* print_year_label[] = {
   /* ENGLISH */        "Print Full Year Calendar",
   /* GERMAN */         "Drucken Jahreskalender",
   /* SPANISH */        "Imprimir Calendario de A\361o",
   /* FRENCH */         "Imprimer Calendrier Annuel",
   /* PORTUGUESE */     "Imprimir Calend\341rio do Ano",
   /* IRISH */          "Print Year Calendar",
   /* ITALIAN */        "Stampare Calendario di Anno"
};

static char* print_month_label[] = {
   /* ENGLISH */        "Print Month Calendar",
   /* GERMAN */         "Monatskalender drucken",
   /* SPANISH */        "Imprimir Calendario de Mes",
   /* FRENCH */         "Imprimer Calendrier du Mois",
   /* PORTUGUESE */     "Imprimir Calend\341rio de M\352s",
   /* IRISH */          "Print Month Calendar",
   /* ITALIAN */        "Stampare Calendario di Mese"
};

static char* print_headers_label[] = {
   /* ENGLISH */        "Print Calendar & Month headers",
   /* GERMAN */         "Kalendar & Monats\374berschriften drucken",
   /* SPANISH */        "Imprimir Calendario & Encabezamientos de Mes",
   /* FRENCH */         "Imprimer Calendrier & Ent\352tes du Mois",
   /* PORTUGUESE */     "Imprimir Calend\341rio & Cabe\347alhos de M\352s",
   /* IRISH */          "Print Calendar & Month headers",
   /* ITALIAN */        "Stampare Calendario & Titolos di Mese"
};

static char* print_items_label[] = {
   /* ENGLISH */        "Print Calendar & Month items",
   /* GERMAN */         "Kalendar & Monatseintr\344ge drucken",
   /* SPANISH */        "Imprimir Calendario & Art\355culos de Mes",
   /* FRENCH */         "Imprimer Calendrier & Entr\351es du Mois",
   /* PORTUGUESE */     "Imprimir Calend\341rio & Artigos de M\352s",
   /* IRISH */          "Print Calendar & Month items",
   /* ITALIAN */        "Stampare Calendario & Articoli di Mese"
};

static char* print_diary_label[] = {
   /* ENGLISH */        "Print Diary",
   /* GERMAN */         "Eintr\344ge drucken",
   /* SPANISH */        "Imprimir Diario",
   /* FRENCH */         "Imprimer Agenda",
   /* PORTUGUESE */     "Imprimir Di\341rio",
   /* IRISH */          "Print Diary",
   /* ITALIAN */        "Stampare Diario"
};

static char* edit_template_label[] = {
   /* ENGLISH */        "Edit Templates",
   /* GERMAN */         "Bearbeiten der Zeichenschablone",
   /* SPANISH */        "Editar Templates",
   /* FRENCH */         "Editer Format Horaire",
   /* PORTUGUESE */     "Editar Modelo",
   /* IRISH */          "Edit Templates",
   /* ITALIAN */        "Introdurre Formato dal'orario"
};

static char* remove_template_label[] = {
   /* ENGLISH */        "Remove Templates",
   /* GERMAN */         "Entfernen der Zeichenschablone",
   /* SPANISH */        "Remover Template",
   /* FRENCH */         "Supprimer Format Horaire",
   /* PORTUGUESE */     "Remover Modelo",
   /* IRISH */          "Remove Templates",
   /* ITALIAN */        "Remuovere Formato dal'orario"
};

static char* save_label[] = {
   /* ENGLISH */        "Save",
   /* GERMAN */         "Speichern",
   /* SPANISH */        "Salve",
   /* FRENCH */         "Sauver",
   /* PORTUGUESE */     "Guardar",
   /* IRISH */          "S\341bha\355l",
   /* ITALIAN */        "Salva"
};

static char* quit_label[] = {
   /* ENGLISH */        "Quit",
   /* GERMAN */         "Beenden",
   /* SPANISH */        "Abandone",
   /* FRENCH */         "Quitter",
   /* PORTUGUESE */     "Sair",
   /* IRISH */          "F\341g",
   /* ITALIAN */        "Esci"
};


/*==========================================================================*/
/*                                  Dialogs:                                */
/*==========================================================================*/

static char* first_entry_label[] = {
   /* ENGLISH */        "No previous diary entry exists",
   /* GERMAN */         "Keine vorherigen Eintr\344ge vorhanden",
   /* SPANISH */        "Ningn art\355culo anterior en diario",
   /* FRENCH */         "Aucune entr\351e pr\351c\351dente n'existe",
   /* PORTUGUESE */     "N\343o artigo anterior em diario",
   /* IRISH */          "No previous diary entry exists",
   /* ITALIAN */        "Alcuni articoli precedenti in diario"
};

static char* last_entry_label[] = {
   /* ENGLISH */        "No following diary entry exists",
   /* GERMAN */         "Keine nachfolgenden Eintr\344ge vorhanden",
   /* SPANISH */        "Ning\370n art\355culo posterior en diario",
   /* FRENCH */         "Aucune entr\351e suivante n'existe",
   /* PORTUGUESE */     "N\343o artigo posterior em diario",
   /* IRISH */          "No following diary entry exists",
   /* ITALIAN */        "Alcuni articoli seguenti in diario"
};

static char* enter_date_label[] = {
   /* ENGLISH */        "Enter Date:",
   /* GERMAN */         "Datum eingeben:",
   /* SPANISH */        "Entre Fecha:",
   /* FRENCH */         "Sp\351cifier la Date:",
   /* PORTUGUESE */     "Introduza a Data:",
   /* IRISH */          "Cuir D\341ta Isteach:",
   /* ITALIAN */        "Digita la data:"
};

static char* enter_backward_label[] = {
   /* ENGLISH */        "Number of preceding items:",
   /* GERMAN */         "Anzahl der verhergehenden Eintr\344ge:",
   /* SPANISH */        "N\372mero de los items anteriores:",
   /* FRENCH */         "Nombre d'entr\351es pr\351c\351dentes:",
   /* PORTUGUESE */     "N\372mero de artigos precedentes:",
   /* IRISH */          "Number of preceding items...:",
   /* ITALIAN */        "Numero di articoli precedenti:"
};

static char* enter_forward_label[] = {
   /* ENGLISH */        "Number of following items:",
   /* GERMAN */         "Anzahl der nachfolgenden Eintr\344ge:",
   /* SPANISH */        "N\372mero de los art\355culos siguientes:",
   /* FRENCH */         "Nombre d'entr\351es suivantes:",
   /* PORTUGUESE */     "N\372mero de artigos seguintes:",
   /* IRISH */          "Number of following items:",
   /* ITALIAN */        "Numero di articoli seguenti:"
};

static char* diary_modified_label[] = {
   /* ENGLISH */        "The diary has been modified.\n"
                        "Changes will be lost.",
   /* GERMAN */         "Die Eintr\344ge wurden ver\344ndert.\n"
                        "Alle \304nderungen gehen verloren.",
   /* SPANISH */        "El diario ha sido modificado.\n"
                        "Todo cambio ser\341 perdido.",
   /* FRENCH */         "L'agenda a \351t\351 modifi\351.\n"
                        "Les modifications seront perdues.",
   /* PORTUGUESE */     "O Di\341rio foi modificado.\n"
                        "As altera\231\245es ser\227o perdidas.",
   /* IRISH */          "Claochla\355odh an Dialann.\n"
                        "Beidh athruithe caillte.",
   /* ITALIAN */        "Il diario \350 stato modificato.\n"
                        "Gli aggiornamenti verranno perduti."
};

static char* okay_label[] = {
   /* ENGLISH */        "OK",
   /* GERMAN */         "OK",
   /* SPANISH */        "OK",
   /* FRENCH */         "OK",
   /* PORTUGUESE */     "OK",
   /* IRISH */          "OK",
   /* ITALIAN */        "OK"
};

static char* cancel_label[] = {
   /* ENGLISH */        "Cancel",
   /* GERMAN */         "Abbrechen",
   /* SPANISH */        "Cancele",
   /* FRENCH */         "Annuler",
   /* PORTUGUESE */     "Cancelar",
   /* IRISH */          "Cealaigh",
   /* ITALIAN */        "Annulla"
};
