/*
 * Copyright (c) 1994  Harris Computer Systems Division
 *
 * The X Consortium, and any party obtaining a copy of these files from the X
 * Consortium, directly or indirectly, is granted, free of charge, a full and
 * unrestricted irrevocable, world-wide, paid up, royalty-free, nonexclusive
 * right and license to deal in this software and documentation files (the
 * "Software"), including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons who receive copies from any such party to do so.
 * This license includes without limitation a license to do the foregoing
 * actions under any patents of the party supplying this software to the X
 * Consortium.
 * 
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
 * THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
 * OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
 * TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
 * PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
 * REPAIR OR CORRECTION.
 * 
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
 * ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
 * THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
 * GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
 * USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF
 * DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
 * PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS),
 * EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGES.
 */

#include "language.h"

#define SUNDAY    0
#define MONDAY    1
#define TUESDAY   2
#define WEDNESDAY 3
#define THURSDAY  4
#define FRIDAY    5
#define SATURDAY  6

#define JANUARY     1
#define FEBRUARY    2
#define MARCH       3
#define APRIL       4
#define MAY         5
#define JUNE        6
#define JULY        7
#define AUGUST      8
#define SEPTEMBER   9
#define OCTOBER     10
#define NOVEMBER    11
#define DECEMBER    12

/*==========================================================================*/
/*                          Application Resources:                          */
/*==========================================================================*/

typedef struct {
   String  language;
   String  diaryFile;
   String  templateFile;
   String  editor;
   String  psViewer;
   String  paper;
   String  backward;
   String  forward;
   Boolean noCalendar;
   Boolean noDiary;
   Boolean longOption;
   int     abbreviate;
   Boolean fullYear;
   Boolean oneMonth;
   Boolean european;
} app_data_type, *app_data_ptr_type;

#define XtNlanguage     "language"
#define XtNdiaryFile    "diaryFile"
#define XtNtemplateFile "templateFile"
#define XtNeditor       "editor"
#define XtNpsViewer     "psViewer"
#define XtNpaper        "paper"
#define XtNnoCalendar   "noCalendar"
#define XtNnoDiary      "noDiary"
#define XtNlong         "long"
#define XtNabbreviate   "abbreviate"
#define XtNbackward     "backward"
#define XtNforward      "forward"
#define XtNfullYear     "fullYear"
#define XtNoneMonth     "oneMonth"
#define XtNeuropean     "european"
#define XtNamerican     "emerican"

#define XtCLanguage     "Language"
#define XtCDiaryFile    "DiaryFile"
#define XtCTemplateFile "TemplateFile"
#define XtCEditor       "Editor"
#define XtCPSViewer     "PSViewer"
#define XtCPaper        "Paper"
#define XtCNoCalendar   "NoCalendar"
#define XtCNoDiary      "NoDiary"
#define XtCLong         "Long"
#define XtCAbbreviate   "Abbreviate"
#define XtCBackward     "Backward"
#define XtCForward      "Forward"
#define XtCFullYear     "FullYear"
#define XtCOneMonth     "OneMonth"
#define XtCEuropean     "European"
#define XtCAmerican     "Emerican"

#define offset(field) XtOffset(app_data_ptr_type, field)
XtResource application_resources[] = {
   {
      XtNlanguage,
      XtCLanguage,
      XtRString,
      sizeof(String),
      offset(language),
      XtRImmediate,
      (XtPointer)NULL
   },
   {
      XtNdiaryFile,
      XtCDiaryFile,
      XtRString,
      sizeof(String),
      offset(diaryFile),
      XtRImmediate,
      (XtPointer)NULL
   },
   {
      XtNtemplateFile,
      XtCTemplateFile,
      XtRString,
      sizeof(String),
      offset(templateFile),
      XtRImmediate,
      (XtPointer)NULL
   },
   {
      XtNeditor,
      XtCEditor,
      XtRString,
      sizeof(String),
      offset(editor),
      XtRImmediate,
      (XtPointer)NULL
   },
   {
      XtNpsViewer,
      XtCPSViewer,
      XtRString,
      sizeof(String),
      offset(psViewer),
      XtRImmediate,
      (XtPointer)NULL
   },
   {
      XtNpaper,
      XtCPaper,
      XtRString,
      sizeof(String),
      offset(paper),
      XtRImmediate,
      (XtPointer)NULL
   },
   {
      XtNnoCalendar,
      XtCNoCalendar,
      XtRBoolean,
      sizeof(Boolean),
      offset(noCalendar),
      XtRImmediate,
      (XtPointer)False
   },
   {
      XtNnoDiary,
      XtCNoDiary,
      XtRBoolean,
      sizeof(Boolean),
      offset(noDiary),
      XtRImmediate,
      (XtPointer)False
   },
   {
      XtNlong,
      XtCLong,
      XtRBoolean,
      sizeof(Boolean),
      offset(longOption),
      XtRImmediate,
      (XtPointer)False
   },
   {
      XtNabbreviate,
      XtCAbbreviate,
      XtRInt,
      sizeof(int),
      offset(abbreviate),
      XtRImmediate,
      (XtPointer)0
   },
   {
      XtNbackward,
      XtCBackward,
      XtRString,
      sizeof(String),
      offset(backward),
      XtRImmediate,
      (XtPointer)NULL
   },
   {
      XtNforward,
      XtCForward,
      XtRString,
      sizeof(String),
      offset(forward),
      XtRImmediate,
      (XtPointer)NULL
   },
   {
      XtNfullYear,
      XtCFullYear,
      XtRBoolean,
      sizeof(Boolean),
      offset(fullYear),
      XtRImmediate,
      (XtPointer)False
   },
   {
      XtNeuropean,
      XtCEuropean,
      XtRBoolean,
      sizeof(Boolean),
      offset(european),
      XtRImmediate,
      (XtPointer)True
   },
   {
      XtNoneMonth,
      XtCOneMonth,
      XtRBoolean,
      sizeof(Boolean),
      offset(oneMonth),
      XtRImmediate,
      (XtPointer)False
   }
};


/*==========================================================================*/
/*                                  Bitmaps:                                */
/*==========================================================================*/

#define right_arrow_width 10
#define right_arrow_height 13
static char right_arrow_bits[] = {
   0x00, 0xfc, 0x04, 0xfc, 0x0c, 0xfc, 0x1c, 0xfc, 0x3c, 0xfc, 0x7c, 0xfc,
   0xfc, 0xfc, 0x7c, 0xfc, 0x3c, 0xfc, 0x1c, 0xfc, 0x0c, 0xfc, 0x04, 0xfc,
   0x00, 0xfc};

#define left_arrow_width 10
#define left_arrow_height 13
static char left_arrow_bits[] = {
   0x00, 0x00, 0x80, 0x00, 0xc0, 0x00, 0xe0, 0x00, 0xf0, 0x00, 0xf8, 0x00,
   0xfc, 0x00, 0xf8, 0x00, 0xf0, 0x00, 0xe0, 0x00, 0xc0, 0x00, 0x80, 0x00,
   0x00, 0x00};

#define check_width 9
#define check_height 8
static char check_bits[] = {
   0x00, 0x01, 0x80, 0x01, 0xc0, 0x00, 0x60, 0x00,
   0x31, 0x00, 0x1b, 0x00, 0x0e, 0x00, 0x04, 0x00
};


/*==========================================================================*/
/*                                   Icons:                                 */
/*==========================================================================*/

#define diary_width 43
#define diary_height 45
static char diary_bits[] = {
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xff, 0xff, 0xff, 0xff, 0x00,
   0x40, 0x01, 0x00, 0x00, 0x80, 0x00, 0xa0, 0xff, 0xff, 0xff, 0xbf, 0x00,
   0x50, 0x00, 0x00, 0x00, 0xa0, 0x00, 0xe8, 0xff, 0xff, 0xff, 0xaf, 0x00,
   0x18, 0x00, 0x00, 0x00, 0xa8, 0x00, 0xf8, 0xff, 0xff, 0xff, 0xab, 0x00,
   0x08, 0x00, 0x00, 0x00, 0xaa, 0x00, 0x08, 0x00, 0x00, 0x00, 0xaa, 0x00,
   0x08, 0x00, 0x00, 0x00, 0xaa, 0x00, 0x88, 0x63, 0x2e, 0x22, 0xaa, 0x00,
   0x88, 0x94, 0x24, 0x22, 0xaa, 0x00, 0x88, 0x94, 0x24, 0x14, 0xaa, 0x00,
   0x88, 0xf4, 0x24, 0x08, 0xaa, 0x00, 0x88, 0x94, 0x24, 0x08, 0xaa, 0x00,
   0x88, 0x93, 0xee, 0x09, 0xea, 0x00, 0x08, 0x00, 0x00, 0x00, 0xea, 0x00,
   0x08, 0x00, 0x00, 0x00, 0xfa, 0x00, 0x08, 0x00, 0x00, 0x00, 0xfa, 0x00,
   0x08, 0x00, 0x00, 0xff, 0xff, 0x00, 0x08, 0x00, 0x00, 0xff, 0xff, 0x00,
   0x08, 0x00, 0x00, 0xe7, 0xff, 0x00, 0x08, 0x00, 0x00, 0x03, 0xbf, 0x00,
   0x08, 0x00, 0x00, 0xe7, 0xbf, 0x00, 0x08, 0x00, 0x00, 0xff, 0xaf, 0x00,
   0x08, 0x00, 0x00, 0xff, 0xaf, 0x00, 0x08, 0x00, 0x00, 0x00, 0xaa, 0x00,
   0x08, 0x00, 0x00, 0x00, 0xaa, 0x00, 0x08, 0x00, 0x00, 0x00, 0xaa, 0x00,
   0x88, 0x73, 0xe6, 0x44, 0xaa, 0x00, 0x88, 0x24, 0x29, 0x45, 0xaa, 0x00,
   0x88, 0x24, 0x29, 0x29, 0xaa, 0x00, 0x88, 0x24, 0xef, 0x10, 0xaa, 0x00,
   0x88, 0x24, 0x29, 0x11, 0x6a, 0x00, 0x88, 0x73, 0x29, 0x11, 0x2a, 0x00,
   0x08, 0x00, 0x00, 0x00, 0x1a, 0x00, 0x08, 0x00, 0x00, 0x00, 0x0a, 0x00,
   0x08, 0x00, 0x00, 0x00, 0x06, 0x00, 0xf8, 0xff, 0xff, 0xff, 0x03, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

#define calendar_width 44
#define calendar_height 49
static char calendar_bits[] = {
 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
 0x00,0x00,0x00,0x80,0xff,0xff,0x00,0x00,0x00,0x80,0x00,0x80,0x00,0x00,0x00,
 0xe0,0xff,0xbf,0x00,0x00,0x00,0x20,0x00,0xa0,0x00,0x00,0x00,0xf8,0xff,0xaf,
 0x00,0x00,0x00,0x08,0x00,0xa8,0x00,0x00,0x00,0x08,0x08,0xa8,0x00,0x00,0x00,
 0x08,0x00,0xa8,0x00,0x00,0x00,0x08,0x1c,0xa8,0x00,0x00,0x00,0x08,0x63,0xa8,
 0x00,0x00,0x00,0x88,0xc1,0xa8,0x00,0x00,0x00,0x88,0x80,0xa9,0x00,0x00,0x00,
 0x48,0x80,0xa9,0x06,0x00,0x00,0x08,0x80,0xa9,0x1a,0x00,0x00,0x08,0x80,0xa9,
 0x61,0x00,0x00,0x08,0xc0,0xa8,0x81,0x01,0x00,0x08,0xc0,0xa8,0x00,0x06,0x00,
 0x08,0x60,0xa8,0x00,0x19,0x00,0x08,0x30,0x68,0x00,0x60,0x00,0x08,0x18,0x68,
 0xe0,0x81,0x01,0x08,0x0c,0x28,0x80,0x01,0x01,0x08,0x06,0x28,0xc0,0x80,0x00,
 0x08,0x03,0x18,0xc0,0x80,0x00,0x88,0x01,0x19,0x60,0x40,0x00,0x88,0x00,0x09,
 0x60,0x40,0x00,0xc8,0x80,0x09,0x30,0x20,0x00,0xc8,0xff,0x05,0x30,0x20,0x00,
 0xc8,0xff,0x05,0x18,0x10,0x00,0x08,0x00,0x02,0x18,0x10,0x00,0x08,0x00,0x02,
 0x0c,0x08,0x00,0xf8,0xff,0x01,0x0c,0x08,0x00,0x00,0x00,0x01,0x06,0x04,0x00,
 0x00,0x80,0x00,0x06,0x04,0x00,0x00,0x80,0x00,0x03,0x02,0x00,0x00,0x40,0x60,
 0x03,0x02,0x00,0x00,0xc0,0xe0,0x01,0x01,0x00,0x00,0x00,0x83,0x07,0x01,0x00,
 0x00,0x00,0x0c,0x86,0x00,0x00,0x00,0x00,0x30,0x80,0x00,0x00,0x00,0x00,0xc0,
 0x40,0x00,0x00,0x00,0x00,0x00,0x43,0x00,0x00,0x00,0x00,0x00,0x2c,0x00,0x00,
 0x00,0x00,0x00,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};

/*==========================================================================*/
/*                                    Misc:                                 */
/*==========================================================================*/

#define MAX_WIDGETS 25

/* in milliseconds */
#define ONE_SECOND          (1000) 
#define ONE_MINUTE          (60 * ONE_SECOND)
#define FIVE_MINUTES        (5  * ONE_MINUTE)
#define ONE_HOUR            (60 * ONE_MINUTE)
#define TWENTY_FOUR_HOURS   (24 * ONE_HOUR)

#define DIARY_FILE "/.xdiary"
#define ITEMS_FILE "/.xdiary-items"
#define TEMPLATE_FILE "/.xdiary-template"

#define DAY   (1L<<1)
#define WEEK  (1L<<2)
#define MONTH (1L<<3)
#define YEAR  (1L<<4)
#define ENTRY (1L<<5)

#ifndef MAXPATHLEN
#define MAXPATHLEN 1024
#endif
